C
      PROGRAM AUTOSLIT
C
C     A U T O S L I T
C
C     LRIS Slit-Mask Setup software.  Routine to automatically 
C     assign slits to objects.  Input must be a file of objects
C     in the standard Norris input format and must contain one 
C     record with the coordinates of the center of the field and
C     a priority of 9999.  This program will match slits with 
C     the objects and produce output files for the LRIS slitmask
C     mill machine and the slitmask alignment program.
C
C     External references:
C
C          PGARRO             PGPLOT Package
C          PGASK                 "     "
C          PGBEG                 "     "
C          PGBOX                 "     "
C          PGCURS                "     "
C          PGDRAW                "     "
C          PGEND                 "     "
C          PGERAS                "     "
C          PGIDEN                "     "
C          PGLAB                 "     "
C          PGLINE                "     "
C          PGMOVE                "     "
C          PGNUMB                "     "
C          PGPAGE                "     "
C          PGPT                  "     "
C          PGSCH                 "     "
C          PGSCI                 "     "
C          PGSVP                 "     "
C          PGTEXT                "     "
C          PGWNAD                "     "
C
C
C     COMPILATION:
C     Use the included Makefile to build the program.  Edit the Makefile
C     to indicate the location of the PGPLOT library, then type 'make'.
C
C
C     HISTORY:  ORIGINAL DIC at CIT 6/21/93
C         based on AUTOFID2 software by CRO, MAS, AP, and DIC at CIT 
C                    
C     SEP-1993 - appearance of output plots modified, added N,E arrows and
C                list of objects on side of plot.  X flipped to reflect
C                flip in LRIS CCD amplifiers.  [JLC - CIT]
C     Dec-1993 - subroutine WRITE_SLITS modified to include a correction
C                for the deviation for a constant scale.  [JLC - CIT]
C     Jan-1994 - J.Cohen - fixed PAR statement for H to allow for range
C                -6.0 to 6.0 hours, was 0.0 to 6.0 hours.
C     Mar-1994 - J.Cohen - reversed sign of THETA, so if you input
C                a standard position angle, i.e.angle of slit towards E
C                from N, everything is OK.  Had to reverse THETA internally,
C                then reverse sign again in two output files.  Also fixed
C                theta limits form -180 to 180 in PAR-RDVAL, and made default 
C                air pressure = 486 mm of Hg, which is correct for Mauna Kea.
C                Changed con file so prompt for THETA refers to position
C                angle.
C     Jul-1994 - Serious bug found in routine AD_TO_XY2.  The input to the
C                gnomoic projection must be in arc-sec on the sky.  It was
C                in arc-sec for dec, but not for RA, for the centerfield
C                position only.  The input RA (in radians) is given by the
C                offfset in mm from the optical axis to the center of the
C                LRIS field.  This needs to be divided by cos(dec).  This
C                was giving problems between various position angles, as the
C                off-axis center of the LRIS field was not being calculated
C                correctly and was moving around on the sky.
C     Aug-1994 - Changed routine that writes file for punch (WRITE_SLITS)
C                to write a file sorted in X.  That is the only way the
C                punch machine works now. (J.Cohen)
C     Dec-1994 - J.Cohen - Caltech  Changed HARD_DEVICE to be permanently
C                set to a Postscript printer in portrait mode.  Added one
C                more plot to the hardcopy output.  Moved the location
C                of the call to lengthen the slits to before the hardcopy
C                output, not afterwards, so the full length of the slits
C                is shown in the printed plots.
C     Apr-1995 - Changed the default for NANGLES and NSETUPS to be 1
C                instead of 6.  Changed the output in NWRITE_SETUP
C                to be the centerfield coordinates precessed, but not
C                including refraction.
C     Jun-1995 - Added a scale correction and an offset in Y to try to
C                predict Y more accurately.  Added the position of the
C                occulting bar to the plots.  Added an indication of
C                the version number.  Sorted the punch files in decreasing
C                Y order, and added "-9999,-9999" as first line in punch file.
C     Sep-1996 - corrected a bug in the calls to PGNUMB (the last parameter
C                was an integer and should be an integer variable), used
C                only in subroutine DRAW_HARD.
C     May-1997 - Major rewrite to create Autoslit version 3.0, including
C                tilted slits, arcs, line segment slits, boxes, and normal
C                slits.  Also modified C-based I/O and an output file for
C                the slitmask mill.
C     Aug-1997 - Changed *_WSLIT variables to *_SLIT_WID.         [PLS]
C     Aug-1997 - Replaced several integer 2's with real 2.0's.    [PLS]
C     Sep-1997 - Made globals SLIT_SEP and SLIT_LEN.              [PLS]
C
C
      IMPLICIT NONE

C     File containing fiber, spectrograph and object definitions.
      INCLUDE  'autoslit.inc'
      INCLUDE  'math.inc'

C     Local variables: autoslit parameters
      CHARACTER INPUT*40,OUTPUT*40,OUT2*40,OUT3*40
      INTEGER*4 PRIORITY,RANK_TOP,RANK_BOT
      REAL*4    NEW_EPOCH
      REAL*4    SLIT_WID,SLIT_LEN,SLIT_SEP
      REAL*8    TEMP,PRES,WAVE,LAT,POS_ANGLE,H
      LOGICAL   MIRROR_USE,ENG_TEST,INTERACTIVE,VERBOSE
C     Local variables: graphics
      CHARACTER DEVICE*32,HARD_DEVICE*32
C     Local variables: misc
      REAL*4    OFFSET_X,OFFSET_Y
      REAL*8    H_RAD,LAT_RAD,WAVE_MU
      REAL*8    VP,ZD,N,R,THETA
      LOGICAL   DONE,INITIAL

C     Functions
      LOGICAL   ASSIGN_SLITS

C     Note regarding version
      WRITE(*,13)' ------------------------------------------'
      WRITE(*,13)'               A U T O S L I T             '
      WRITE(*,11) VERSION
 11   FORMAT(' This is version ',F3.1,' with many new features')
      WRITE(*,13)' including abilty to use tilted slits, make'
      WRITE(*,13)' arcs, dual ranks of slits, etc.           '
      WRITE(*,13)' ------------------------------------------'
 13   FORMAT(A)

C     Get user parameters from input file.
      CALL LOAD_PARAMS (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                  RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                  SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE, LAT,
     :                  POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                  INTERACTIVE, VERBOSE, HARD_DEVICE)

C     Print parameters for confirmation.
      CALL PRINT_PARAMS (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                   RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                   SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                   LAT, POS_ANGLE, H, MIRROR_USE, ENG_TEST,
     :                   INTERACTIVE, VERBOSE, STDL, STDL, STDOUT)

C     Get object data from input file.
      CALL GET_DATA(INPUT)

C     Match special slits with objects and sort list.
      CALL MATCH_SLITS(MIRROR_USE)

C     Now, we start the real work:
C     Precess slit coordinates to NEW_EPOCH.
      CALL PRECESS(NEW_EPOCH)

C     Correct slit coordinates for atmospheric refraction.
      H_RAD = H * RPH
      LAT_RAD = LAT * RPD
      WAVE_MU = WAVE / 10000.0D0
      CALL REFRACT(H_RAD,LAT_RAD,PRES,TEMP,WAVE_MU,VP,ZD,N,R)

C     Determine location of slits in X-Y mask space.
      THETA = -POS_ANGLE               ! internal theta opposite
      CALL XYPOS(THETA)

C     Adjust positions to place guide star in pickoff mirror.
      IF (MIRROR_USE) THEN
         OFFSET_X = OBJ_T_X(GUIDE_OBJ) - G_X
         OFFSET_Y = OBJ_T_Y(GUIDE_OBJ) - G_Y
         CALL XYSHIFT(OFFSET_X,OFFSET_Y,THETA)
      END IF

C     Primary slit-assignment loop.
C     The existence of this loop allows interactive changes
C     to the field position angle, small field offsets, etc.
      DONE = .FALSE.
      INITIAL = .TRUE.   ! first iteration
      DO WHILE (.NOT. DONE)
C        Eliminate objects not inside the mask region.
         CALL XYINMASK(VERBOSE)

C        Create output file of all objects in mask.
         CALL WRITE_OBJECTS (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                       RANK_TOP, RANK_BOT, NEW_EPOCH,
     :                       SLIT_WID, SLIT_LEN, SLIT_SEP,
     :                       TEMP, PRES, WAVE, LAT, POS_ANGLE, H,
     :                       MIRROR_USE, ENG_TEST, INTERACTIVE)

C        Assign slits to mask.
         DONE = ASSIGN_SLITS (OUTPUT, INTERACTIVE, DEVICE,
     :                        HARD_DEVICE, PRIORITY, THETA,
     :                        SLIT_WID, SLIT_LEN, SLIT_SEP,
     :                        MIRROR_USE, ENG_TEST, VERBOSE,
     :                        INITIAL)

         POS_ANGLE = -THETA     ! in case value changed interactively
         INITIAL = .FALSE.
      END DO

C     Write output setup file.
      CALL WRITE_SETUP(INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                 RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                 SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                 LAT, POS_ANGLE, H, MIRROR_USE,
     :                 ENG_TEST, INTERACTIVE)

C     Write output file for punch/mill machine.
      CALL WRITE_MILL (INPUT, OUTPUT, OUT2, OUT3, PRIORITY,
     :                 RANK_TOP, RANK_BOT, NEW_EPOCH, SLIT_WID,
     :                 SLIT_LEN, SLIT_SEP, TEMP, PRES, WAVE,
     :                 LAT, POS_ANGLE, H, THETA, MIRROR_USE,
     :                 ENG_TEST, INTERACTIVE)

      END
