/* File: get_key.c */
/* Description: Use string routines to parse keywords and */
/*              values from the autoslit parameter file.  */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>


#define	COMMCHAR	'#'		/* Comment line character   */
#define RPD		M_PI / 180.	/* Radians per degree       */
#define WHITESPACE	" \t"		/* White space for strtok() */

#define	DEFAULT	0	     		/* Defaulted parameter value. Must */
					/*   match value in autoslit.inc.  */
#define ERROR		-1
#define FALSE		0
#define TRUE		1


/* Parse a keyword/value line. */
/* Return the value in four different data types. */
void get_key_ (line, key, nkey, cval, nval, ival, rval, dval)
char *line;				/* Input line         */
char *key;				/* Output keyword     */
int *nkey;				/* Length of keyword  */
char *cval;				/* Key value: string  */
int *nval;				/* Length of cval     */
int *ival;				/* Key value: integer */
float *rval;				/* Key value: real    */
double *dval;				/* Key value: double  */
{
  char *cptr;
  int i;

  /* Extract keyword. */
  if (((cptr = strtok (line, WHITESPACE)) == NULL) || (*cptr == COMMCHAR)) {
    *nkey = 0;
    return;				/* Blank line or comment */
  }
  (void) strcpy (key, cptr);
  *nkey = strlen (key);
  for (i = 0; i < (*nkey); i++)
    key[i] = toupper(key[i]);		/* Keyword returned in upper-case */

  /* Extract key value. */
  if ((cptr = strtok (NULL, WHITESPACE)) == NULL) {
    *nval = 0;
    return;				/* No key value (e.g., 'END') */
  }
  (void) strcpy (cval, cptr);
  *nval = strlen (cval);
  /*  for (i = 0; i < (*nval); i++) */
  /*    cval[i] = tolower(cval[i]);	/* Key value returned in lower-case */

  /* Attempt conversion of key value. */
  (void) sscanf (cval, "%d", ival);	/* Attempt integer conversion */
  (void) sscanf (cval, "%f", rval);	/* Attempt real conversion    */
  (void) sscanf (cval, "%lf", dval);	/* Attempt double conversion  */

  return;
}


/* Parse a line describing a tilted slit. */
/* Return <0 if error, 0 if blank line, and length(name) if successful. */
int get_tilt_ (line, name, angle, up, down)
char *line;				/* Input line      */
char *name;				/* Object for slit */
float *angle;				/* Slit angle      */
float *up;				/* Min extent up   */
float *down;				/* Min extent down */
{
  char *cptr;

  /* Extract name. */
  if (((cptr = strtok (line, WHITESPACE)) == NULL) || (*cptr == COMMCHAR))
    return (0);				/* Blank line or comment */
  (void) strcpy (name, cptr);

  /* Extract numerical values. */
  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No angle */
  (void) sscanf (cptr, "%f", angle);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No min extent up */
  (void) sscanf (cptr, "%f", up);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No min extent down */
  (void) sscanf (cptr, "%f", down);

  return (strlen(name));
}


/* Parse a line describing an arc slit. */
/* Return <0 if error, 0 if blank line, and length(name) if successful. */
int get_arc_ (line, name, ra, dec, epoch, radius, minpa, maxpa)
char *line;				/* Input line         */
char *name;				/* Arc slit name      */
double *ra, *dec;			/* Position of arc    */
float *epoch;				/* Epoch of position  */
float *radius;				/* Radius of arc      */
float *minpa;				/* Min position angle */
float *maxpa;				/* Max position angle */
{
  float rah, ram, ras;		/* Right ascension components    */
  float decd, decm, decs;	/* Declination components        */
  char *cptr, *sign;

  /* Extract name. */
  if (((cptr = strtok (line, WHITESPACE)) == NULL) || (*cptr == COMMCHAR))
    return (0);				/* Blank line or comment */
  (void) strcpy (name, cptr);

  /* Extract numerical values. */
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &rah);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ram);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ras);
  *ra = (double) ((rah + (ram/60.) + (ras/3600.)) * 15. * RPD);

  sign = strtok (NULL, WHITESPACE);
  (void) sscanf (sign, "%f", &decd);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decm);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decs);
  if (*sign == '-')
    *dec = (double) ((decd - (decm/60.) - (decs/3600.)) * RPD);
  else
    *dec = (double) ((decd + (decm/60.) + (decs/3600.)) * RPD);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No epoch */
  (void) sscanf (cptr, "%f", epoch);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No radius */
  (void) sscanf (cptr, "%f", radius);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No min position angle */
  (void) sscanf (cptr, "%f", minpa);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No max position angle */
  (void) sscanf (cptr, "%f", maxpa);

  return (strlen(name));
}


/* Parse a line describing a line segment slit. */
/* Return <0 if error, 0 if blank line, and length(name) if successful. */
int get_line_ (line, name, ra, dec, epoch)
char *line;				/* Input line         */
char *name;				/* Segment slit name  */
double *ra, *dec;			/* Position of arc    */
float *epoch;				/* Epoch of position  */
{
  float rah, ram, ras;		/* Right ascension components    */
  float decd, decm, decs;	/* Declination components        */
  char *cptr, *sign;

  /* Extract name. */
  if (((cptr = strtok (line, WHITESPACE)) == NULL) || (*cptr == COMMCHAR))
    return (0);				/* Blank line or comment */
  (void) strcpy (name, cptr);

  /* Extract numerical values. */
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &rah);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ram);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ras);
  *ra = (double) ((rah + (ram/60.) + (ras/3600.)) * 15. * RPD);

  sign = strtok (NULL, WHITESPACE);
  (void) sscanf (sign, "%f", &decd);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decm);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decs);
  if (*sign == '-')
    *dec = (double) ((decd - (decm/60.) - (decs/3600.)) * RPD);
  else
    *dec = (double) ((decd + (decm/60.) + (decs/3600.)) * RPD);

  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    return (ERROR);			/* No epoch */
  (void) sscanf (cptr, "%f", epoch);

  return (strlen(name));
}


/* Parse a line describing a box slit. */
/* Return <0 if error, 0 if blank line, and length(name) if successful. */
int get_box_ (line, name, size)
char *line;				/* Input line      */
char *name;				/* Object for slit */
float *size;				/* Box size        */
{
  char *cptr;

  /* Extract name. */
  if (((cptr = strtok (line, WHITESPACE)) == NULL) || (*cptr == COMMCHAR))
    return (0);				/* Blank line or comment */
  (void) strcpy (name, cptr);

  /* Extract numerical values. */
  if ((cptr = strtok (NULL, WHITESPACE)) == NULL)
    size = DEFAULT;			/* Box size optional */
  else
    (void) sscanf (cptr, "%f", size);

  return (strlen(name));
}


