/* File: get_obj.c */
/* Description: Use string routines to parse object */
/*              info from the autoslit object file. */
/*                                                                */
/* The input records contain the following information            */
/* (in this required order):                                      */
/*                                                                */
/*   OID Pr Mag RAh RAm RAs DECd DECm DECs Ep Eq RAPM DECPM       */
/*                                                                */
/*    where                                                       */
/*                                                                */
/*   OID   = Object identifier (or name)                          */
/*   Pr    = Object priority as assigned by the observer          */
/*   Mag   = Object magnitude                                     */
/*   RAh   = Object right ascension hours                         */
/*   RAm   = Object right ascension minutes                       */
/*   RAs   = Object right ascension seconds                       */
/*   DECd  = Object declination degrees                           */
/*   DECm  = Object declination minutes                           */
/*   DECs  = Object declination seconds                           */
/*   Ep    = Epoch of the object's coordinates                    */
/*   Eq    = Object equinox                                       */
/*   RAPM  = Object proper motion in right ascension in arcsec/yr */
/*   DECPM = Object proper motion in declination in arcsec/yr     */
 
#include <stdio.h>
#include <string.h>
#include <math.h>


#define NULLOBJ		" "		/* Null object name         */
#define RPD		M_PI / 180.	/* Radians per degree       */
#define WHITESPACE	" \t"		/* White space for strtok() */
#define MAXLINE		255		/* Maximum length of line   */

#define ERROR		-1
#define FALSE		0
#define TRUE		1


/* Parse an object line. */
/* Return <0 if error, 0 if blank line, and length(obj) if successful. */
int get_obj_ (line, obj, ra, dec, dra, ddec, epoch, equinox, priority, mag)
char *line;			/* Input line                    */
char *obj;			/* Object name                   */
double *ra, *dec;		/* Object position [rad]         */
float *dra, *ddec;		/* Object proper motion ["/year] */
float *epoch, *equinox;		/* Coordinate epoch              */
int *priority;			/* Object priority               */
float *mag;			/* Object magnitude              */
{
  float rah, ram, ras;		/* Right ascension components    */
  float decd, decm, decs;	/* Declination components        */
  char lbuf[MAXLINE];		/* Temporary for parsing tokens  */
  char *sign;			/* Temporary to check sign of dec*/
  int ntok;

  /* Count tokens in line. */
  (void) strcpy (lbuf, line);
  if (strtok (lbuf, WHITESPACE) == NULL)
    return (0);				/* No tokens (i.e., blank line) */
  ntok = 1;
  while (strtok (NULL, WHITESPACE) != NULL)
    ntok++;

  /* Too few tokens -> error. */
  if (ntok < 12)
    return (ERROR);

  /* Determine if object name provided. */
  (void) strcpy (lbuf, line);		/* Do not corrupt line string. */
  if (ntok == 12) {
    (void) strcpy (obj, NULLOBJ);
    (void) sscanf (strtok (lbuf, WHITESPACE), "%d", priority);
  } else {
    (void) strcpy (obj, strtok (lbuf, WHITESPACE));
    (void) sscanf (strtok (NULL, WHITESPACE), "%d", priority);
  }

  /* Extract rest of numerical values. */
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", mag);

  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &rah);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ram);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &ras);
  *ra = (double) ((rah + (ram/60.) + (ras/3600.)) * 15. * RPD);

  sign = strtok (NULL, WHITESPACE);
  (void) sscanf (sign, "%f", &decd);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decm);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", &decs);
  if (*sign == '-')
    *dec = (double) ((decd - (decm/60.) - (decs/3600.)) * RPD);
  else
    *dec = (double) ((decd + (decm/60.) + (decs/3600.)) * RPD);

  (void) sscanf (strtok (NULL, WHITESPACE), "%f", epoch);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", equinox);

  (void) sscanf (strtok (NULL, WHITESPACE), "%f", dra);
  (void) sscanf (strtok (NULL, WHITESPACE), "%f", ddec);

#ifdef DEBUG
  (void) printf ("Read: %s %lf %lf %f %f %f %f %d %f\n", obj, *ra, *dec, *dra, *ddec, *epoch, *equinox, *priority, *mag);
#endif

  return (strlen (obj));
}

