C
C     Refract slit coordinates to specified hour angle.
C     Borrowed from CRO's precession program.
C
      SUBROUTINE REFRACT(H,LAT,PRES,TEMP,WAVE,VP,ZD,N,R)
C
C     Routine which does the refraction.
C     Parameters:
C     H    -> Hour angle to which to refract (radians)
C     LAT  -> Latitude of the observatory (radians)
C     PRES -> Pressure of the atmosphere (mm of Hg)
C     Wave -> Wavelength to use to precess (microns)
C     VP   -> not used
C     ZD   -> Zenith distance (computed)
C     N    -> index of refraction (computed)
C     R    -> refraction coefficient (computed)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      REAL*8    H,LAT,PRES,TEMP,WAVE,VP,ZD,N,R

C     Local variables
      REAL*8    HA
      INTEGER*4 I

      WRITE(*,*)''
      WRITE(*,11)' Refracting coordinates...'
 11   FORMAT(A)

C     Refract slit coordinates.
C     Overwrite original object coordinates with precessed+refracted ones.
      DO I=1,NPTS
         HA = H + DRA0-RAS(I)      ! use original coords to find delta-H
         CALL RAD_REFRACTION(RAREFR(I),DECREFR(I),RAPREC(I),DECPREC(I),
     :                       HA,LAT,PRES,TEMP,WAVE,VP,ZD,N,R)
      END DO

C     Refract the center coordinate.
C     Overwrite original object coordinates with precessed+refracted ones.
      HA = H
      CALL RAD_REFRACTION(DRA0REFR,DDEC0REFR,DRA0PREC,DDEC0PREC,
     :                    HA,LAT,PRES,TEMP,WAVE,VP,ZD,N,R)

C     Clean up and quit.
      WRITE(*,101) NUSE
 101  FORMAT(' Proceeding with ',I4,' positions...')
      RETURN
      END


C
C     Refraction subroutine borrowed from Norris spectrograph.
C
      SUBROUTINE RAD_REFRACTION(ARA,ADEC,RA,DEC,
     :                          H,LAT,PRES,TEMP,WAVE,VP,ZD,N,R)
C
C     Calculate the apparent declination and right ascention corrected
C     for atmospheric refraction, given the true RA and DEC, etc.
C     RA and DEC are in radians.
C
C     Parameters -    (">" input, "<" output)
C
C     (<) ARA        (Real*8) Apparent Right Ascention in radians
C
C     (<) ADEC       (Real*8) Apparent Declination in radians
C  
C     (>) RA         (Real*8) True Right Ascention in radians
C
C     (>) DEC        (Real*8) True Declination in radians
C
C     (>) H          (Real*8) Hour angle in radians
C  
C     (>) LAT        (Real*8) Observer's latitude in radians
C
C     (>) PRES       (Real*8) Local air pressure in mm Hg.
C
C     (>) TEMP       (Real*8) Local air temperature in Celsius.
C   
C     (>) WAVE       (Real*8) Wavelength in micrometers
C
C     (>) VP         (Real*8) Vapor pressure in mm Hg (not currently used.)
C
C     (<) ZD         (Real*8) Zenith distance in degrees
C
C     (<) N          (Real*8) Atmospheric refractivity
C
C     (<) R          (Real*8) Refraction coeficient
C
C     No external references.
C
C     History:  CRO/CIT.  20 June 1988.  Original.
C
      IMPLICIT NONE

      INCLUDE 'math.inc'

C     Formal parameters
      REAL*8 ARA,ADEC,RA,DEC,H,LAT,PRES,TEMP,WAVE,VP,ZD,N,R

C     Local variables
      REAL*8 DA,DD,NUM,TCORR,WAVERS,SINA,SINQ,COSQ,SINZ,COSZ,TANZ
C     REAL*8 A,Z,Q,AD

C     Calculate altitude and zenith distance.
      SINA = DSIN(LAT) * DSIN(DEC) + DCOS(LAT) * DCOS(DEC) * DCOS(H)
      COSZ = SINA
      SINZ = DSQRT (1 - SINA * SINA)
      TANZ = SINZ / COSZ

C     A = DSIN(LAT)*DSIN(DEC) + DCOS(LAT)*DCOS(DEC)*DCOS(H)
C     A = DASIN(A)
C     Z = HALFPI - A
C
C     Parallactic angle, q
C
C     Q = DASIN( DCOS(LAT)*DSIN(H)/DSIN(Z) )

      IF (SINZ .NE. 0.0) THEN
C        Object on zenith.
         SINQ = DCOS(LAT) * DSIN(H) / SINZ
         COSQ = (DCOS (HALFPI - LAT) - COSZ * DCOS(HALFPI - DEC))/
     :          (SINZ * DSIN(HALFPI - DEC))
      ELSE
C        Zenith distance is 0 so Q doesn't matter. R=0 anyway.
         SINQ = 0.0
         COSQ = 0.0
      END IF

C      check quadrant of q - note that correct formula for
C          cos(q) is expression below divided by
C          sin(z)*sin(ninety-dec), but only numerator
C          suffices to get sign of cos(q).
C
C        cosq = (dcos(halfpi-lat) - dcos(z)*dcos(halfpi-dec))
C          if (cosq.lt.0.0)  then
C                q = pi-q             
c             type *,' q in second quadrant,dec,ha ',dec,h
C         end if

C     Calculate refractive index at standard TP.
      WAVERS = 1. / (WAVE * WAVE)
      N = 64.328 + (29498.1/(146.-WAVERS)) + (255.4/(41.-WAVERS)) 
      N = 1.E-6 * N
      

C     Calculate temperature and pressure correction.
      TCORR = 1. + 0.003661 * TEMP
      NUM = 720.88 * TCORR
      N = N * ((PRES * (1.+(1.049-0.0157*TEMP) * 1.E-6 * PRES)) / NUM)
      
C     Calculate correction for water vapor pressure
C     N = N - (((0.0624-0.000680*WAVERS)/TCORR)*VP)

C     AD = A * DPR
      ZD = DASIN(SINZ) * DPR

C     Calculate refraction (R).
      R = N * 206265. * TANZ

C     Calculate changes in RA and DEC in radians.
      DA = R * SINQ * RPAS / DCOS(DEC)
      DD = R * COSQ * RPAS

C     WRITE(*,*)'COS(Z) SIN(Z) = ',COSZ,SINZ
C     WRITE(*,*)'COS(Q) SIN(Q) = ',COSQ,SINQ
C     WRITE(*,*)'DA,DD = ',DA,DD

C     Correct RA and DEC.
      ARA =  RA +  DA
      ADEC = DEC + DD

      RETURN

      END
