C
C     Compute the (X,Y) coordinates for each slit in the mask.
C
      SUBROUTINE XYPOS(THETA)

      IMPLICIT  NONE

      include 'autoslit.inc'

C     Formal parameters
      REAL*8    THETA

C     Local variables
      INTEGER*4 I,NSTAT
      REAL*8    XX,YY

      WRITE(*,*)''
      WRITE(*,11)' Computing mask coordinates...'
 11   FORMAT(A)

C     Compute mask coordinates.
      DO I=1,NPTS
         CALL AD_TO_XY(DRA0REFR,DDEC0REFR,RAREFR(I),DECREFR(I),
     :                 XX,YY,THETA,NSTAT)
         IF (NSTAT .LT. 0) THEN
            CALL USER_ERR('Transformation error too large: '//
     :                     ONAMES(I),70)
         END IF

         OBJ_T_X(I) = XX
         OBJ_T_Y(I) = YY
      END DO

C     Clean up and quit.
      WRITE(*,101) NUSE
 101  FORMAT(' Proceeding with ',I4,' positions...')
      RETURN

      END


C
C     Shift all mask coordinates by specified offset.
C
      SUBROUTINE XYSHIFT(OFFSET_X,OFFSET_Y,THETA)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      REAL*4    OFFSET_X,OFFSET_Y
      REAL*8    THETA

C     Local variables
      REAL*8    ANGLE
      INTEGER*4 I

      WRITE(*,*)''
      WRITE(*,11)' Shifting coordinates...'
 11   FORMAT(A)

C     Adjust all coordinates by offset.
      DO I = 1,NPTS
         OBJ_T_X(I) = OBJ_T_X(I) - OFFSET_X
         OBJ_T_Y(I) = OBJ_T_Y(I) - OFFSET_Y
      END DO

C     Adjust (RA,DEC) coordinates of center of field.
C     Note: moving reference point, not object, so *add* offset.
C     Note: must do both precessed and precessed+refracted.
      ANGLE = THETA * RPD
      DRA0PREC  = DRA0PREC + RPAS * (DCOS(ANGLE) * OFFSET_X -
     :                       DSIN(ANGLE) * OFFSET_Y) / SCALE
      DDEC0PREC = DDEC0PREC + RPAS * (DSIN(ANGLE) * OFFSET_X +
     :                        DCOS(ANGLE) * OFFSET_Y) / SCALE
      DRA0REFR  = DRA0REFR + RPAS * (DCOS(ANGLE) * OFFSET_X -
     :               DSIN(ANGLE) * OFFSET_Y) / SCALE
      DDEC0REFR = DDEC0REFR + RPAS * (DSIN(ANGLE) * OFFSET_X +
     :                DCOS(ANGLE) * OFFSET_Y) / SCALE

C     Clean up and quit.
      WRITE(*,101) NUSE
 101  FORMAT(' Proceeding with ',I4,' slits...')
      RETURN

      END


C
C     Eliminate objects not inside the mask region.
C
      SUBROUTINE XYINMASK(VERBOSE)

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'

C     Formal parameters
      LOGICAL VERBOSE

C     Local variables
      INTEGER*4 I

      WRITE(*,*)''
      WRITE(*,11)' Checking coordinates with mask...'
 11   FORMAT(A)

C     Calculate size of slit mask area to use.
      XSIDE = XS_SLIT
      IF (XS_SLIT .GT. (CCD_SIDE * CCD_SCALE))
     :   XSIDE = CCD_SIDE * CCD_SCALE
      YSIDE = YS_SLIT
      IF (YS_SLIT .GT. (CCD_SIDE * CCD_SCALE)) 
     :     YSIDE = CCD_SIDE * CCD_SCALE

C     Check that each object is within the mask.
      NUSE = NPTS
      DO I=1,NPTS
         OBJ_OUT(I) = .FALSE.
         OBJ_HIT(I) = .FALSE.
         OBJ_SLIT(I) = 0
         IF (OBJ_T_X(I) .GT. X_CENTER + (XSIDE/2.0) .OR.
     :       OBJ_T_X(I) .LT. X_CENTER - (XSIDE/2.0) .OR.
     :       OBJ_T_Y(I) .GT. Y_CENTER + (YSIDE/2.0) .OR.
     :       OBJ_T_Y(I) .LT. Y_CENTER - (YSIDE/2.0)) THEN
            OBJ_OUT(I) = .TRUE.
            NUSE = NUSE - 1
            IF (VERBOSE) THEN
               WRITE(*,51)I,ONAMES(I)
 51            FORMAT(' Eliminating object #',I4,': ',A)
            END IF
         END IF
      END DO

C     Clean up and quit.
      WRITE(*,101) NUSE
 101  FORMAT(' Proceeding with ',I4,' positions...')
      RETURN

      END


C
C     Transform (RA,DEC) coordinates to (X,Y) mask coordinates.
C
      SUBROUTINE AD_TO_XY(RA0,DEC0,RA,DEC,X,Y,THETA,STATUS)
CC
C     Subroutine to transform a coordinate pair on the celestial
C     sphere (RA, DEC) to an x,y coordinate on a plate at the 
C     Cassegrain focus of the Keck.  
C
C     Parameters: 	( ">" = input, "<" = output)
C
C     (>) RA0      (Real*8) Right ascension of the field center in radians.
C
C     (>) DEC0     (Real*8) Declination of the field center, in radians.
C
C     (>) RA       (Real*8) Right ascension of the object, in radians.
C     
C     (>) DEC      (Real*8) Declination of the object, in radians.
C
C     (<) X        (Real*8) The x coordinate, in mm, of the object as
C                   projected on to the Norris xy stage.
C
C     (<) Y        (Real*8) The y coordinate, in mm, of the object as
C                  projected on to the Norris xy stage.
C
C     (>) THETA    (Real*8) The position angle of the mask with respect
C                  to the Declination and Right Ascension lines.
C
C     (>) STATUS   (Integer*4)  Error return.
C
C     The transformation is based on algorithms found in the program ASTRO, 
C     which uses two equations linear in scale factors with radial distortion 
C     factors in r**2, r**4 and r**6.  We only use one radial distortion term.
C     This transform adds scale and distortion parameters to the normal
C     gnomic projection:
C
C                         cos(DEC)sin(RA-RA0)
C       ETA = -----------------------------------------------
C             sin(DEC)sin(DEC0) + cos(DEC)cos(DEC0)cos(RA-RA0)
C
C             sin(DEC)cos(DEC0) - cos(DEC)sin(DEC0)cos(RA-RA0)
C        NU = ------------------------------------------------
C             sin(DEC)sin(DEC0) + cos(DEC)cos(DEC0)cos(RA-RA0)
C
C     where
C
C            ETA  = Standard x coordinate on the plate
C             NU  = Standard y coordinate on the plate
C             RA  = right ascension of an object
C            DEC  = declination of an object
C            RA0  = right ascension of the tangent point to the
C                   celestial spere.
C           DEC0  = declination of the tangent point.
C
C    Instead of multiplying the above equation by the focal length, the
C    scale and distortions are applied by solving the following equations
C    numerically for dx and dy, the offsets in mm from the center of the
C    plate:
C
C       ETA  = v(1) + [ cos(v(3))dx+sin(v(3))dy][1+v(4)r]
C       NU   = v(2) + [-sin(v(3))dx+cos(v(3))dy][1+v(4)r]
C
C     where
C
C		v(1) = plate center in RA (RA0)
C  		v(2) = plate center in DEC (DEC0)
C 		v(3) = angle between x axis and -RA axis(deg)
C    		v(4) = radial distortion of the telescope
C
C     External references: 
C
C          READ_PARAMS            Norris Library
C          TO_TABLE2                 "      "
C          NL_TO_TABLE2              "      "
C
C     History:
C		CRO/CIT.  11 May 1989.  Original.
C               CRO/CIT   27 May 1989.  Added gnomic projection.
C               DIC/CIT   21 June 1993. Changed to work on the LRIS.
C               J.Cohen/CIT July 1994   Fixed bug - RA needs to be in
C                                       arc-sec of RA, not just radians.
C                                       The 1 line changed is marked.

      IMPLICIT NONE

      INCLUDE 'autoslit.inc'
      INCLUDE 'math.inc'

C     Formal parameters
      REAL*8    RA0,DEC0,RA,DEC,X,Y,THETA
      INTEGER*4 STATUS

C     Local variables
      REAL*8    ETA,NU,Y_CEN,X_CEN
      REAL*8    RAC,DECC,RA1,DEC1
      REAL*8    DR,SDR,CDR,SD,CD,SD0,CD0,DENOM,XO
      REAL*8    ANGLE,M_ANGLE,BEND

C     Not really checking for problems...
      STATUS = 0

C     Assign the field center RA,DEC in arcseconds.
      M_ANGLE = MASK_ANGLE * RPD
      BEND = MASK_BEND * RPD
      ANGLE = THETA * RPD
      XO = X_CENTER / SCALE

C     The following line was changed. [JLC]
C     RA1 = XO * DCOS(ANGLE) * RPAS
      RA1 = XO * DCOS(ANGLE) * RPAS / DCOS(DEC0)
      DEC1 = XO * DSIN(ANGLE) * RPAS
      RAC = RA0 - RA1
      DECC = DEC0 - DEC1

      X_CEN = RAC / RPAS
      Y_CEN = DECC / RPAS

C     Compute gnomonic projection.
      DR = RA - RAC
      SDR = DSIN(DR)
      CDR = DCOS(DR)
      SD = DSIN(DEC)
      CD = DCOS(DEC)
      SD0 = DSIN(DECC)
      CD0 = DCOS(DECC)
      DENOM = SD * SD0 + CD * CD0 * CDR

C     Calculate normal coordinates ETA,NU and convert to arcseconds.
      ETA=  CD * SDR / (DENOM * RPAS)
      NU = (SD * CD0 - CD * SD0 * CDR) / (DENOM * RPAS)

C     Transform the normal coordinates ETA,NU
C     to table coordinates on the stage X,Y.
      X = ( DCOS(ANGLE) * ETA + DSIN(ANGLE) * NU) * SCALE
      Y = (-DSIN(ANGLE) * ETA + DCOS(ANGLE) * NU) * SCALE

C     Transform to final mask coordinates.
      X = X / DCOS(M_ANGLE)
      Y = Y / DCOS(BEND)

      RETURN

      END
